'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _MatchingConfig = require('./config/MatchingConfig');

var _MatchingConfig2 = _interopRequireDefault(_MatchingConfig);

var _IndexingConfig = require('./config/IndexingConfig');

var _IndexingConfig2 = _interopRequireDefault(_IndexingConfig);

var _AggregatingConfig = require('./config/AggregatingConfig');

var _AggregatingConfig2 = _interopRequireDefault(_AggregatingConfig);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var TabulationQueryBuilder = function () {
  function TabulationQueryBuilder() {
    var config = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    (0, _classCallCheck3.default)(this, TabulationQueryBuilder);
    this.config = {};
    this.indexingConfigList = [];

    this.config = config;
  }

  TabulationQueryBuilder.prototype.setTable = function setTable(table) {
    this.table = table;
  };

  TabulationQueryBuilder.prototype.setMatching = function setMatching(config) {
    this.matchingConfig = new _MatchingConfig2.default((0, _extends3.default)({}, config, this.config));
  };

  TabulationQueryBuilder.prototype.setIndexing = function setIndexing(config) {
    this.indexingConfigList[0] = new _IndexingConfig2.default((0, _extends3.default)({}, config, this.config));
  };

  TabulationQueryBuilder.prototype.setIndexings = function setIndexings(configs) {
    var _this = this;

    this.indexingConfigList = configs.map(function (config, index) {
      return new _IndexingConfig2.default((0, _extends3.default)({}, config, _this.config, { index: index }));
    });
  };

  TabulationQueryBuilder.prototype.setAggregating = function setAggregating(config) {
    this.aggregatingConfig = new _AggregatingConfig2.default((0, _extends3.default)({}, config, this.config));
  };

  TabulationQueryBuilder.prototype.build = function build() {
    if (this.aggregatingConfig.method === 'count') {
      return this.buildWithIndexing();
    }

    var matchingTable = this.matchingConfig.build(this.table, this.indexingConfigList, this.aggregatingConfig).toString();

    return this.aggregatingConfig.build(this.addParen(matchingTable), this.indexingConfigList).toString();
  };

  TabulationQueryBuilder.prototype.buildWithIndexing = function buildWithIndexing() {
    var matchingTable = this.matchingConfig.build(this.table, this.indexingConfigList, this.aggregatingConfig).toString();

    var indexingTable = _IndexingConfig2.default.build(this.addParen(matchingTable), this.aggregatingConfig, this.indexingConfigList, this.config.segment).toString();

    return this.aggregatingConfig.build(this.addParen(indexingTable), this.indexingConfigList.map(function (config) {
      return {
        field: config.field,
        interval: config.interval,
        categoryRange: config.categoryRange
      };
    })).toString();
  };

  TabulationQueryBuilder.prototype.addParen = function addParen(str) {
    return '(' + str + ')';
  };

  return TabulationQueryBuilder;
}();

exports.default = TabulationQueryBuilder;