'use strict';

var _ava = require('ava');

var _ava2 = _interopRequireDefault(_ava);

var _TabulationQueryBuilder = require('./TabulationQueryBuilder');

var _TabulationQueryBuilder2 = _interopRequireDefault(_TabulationQueryBuilder);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

(0, _ava2.default)('User count by payment amount', function (t) {
  var tqb = new _TabulationQueryBuilder2.default();

  tqb.setTable('payment_logs');

  tqb.setMatching({
    field: 'timestamp',
    range: ['2017-01-01 00:00:00', '2017-01-03 23:59:59']
  });

  tqb.setAggregating({
    field: 'user_id',
    method: 'count'
  });

  tqb.setIndexing({
    field: 'price',
    method: 'sum',
    interval: 300
  });

  t.is(tqb.build(), 'SELECT COUNT(`user_id`) AS "value", FLOOR(`indexed_value_0` / 300) AS "category" FROM (SELECT SUM(`indexed_value_0`) AS "indexed_value_0", `user_id` FROM (SELECT price AS "indexed_value_0", user_id FROM payment_logs WHERE ("2017-01-01 00:00:00" <= timestamp AND timestamp <= "2017-01-03 23:59:59")) `matching_table` GROUP BY `user_id`) `indexing_table` GROUP BY FLOOR(`indexed_value_0` / 300)');
});

(0, _ava2.default)('Total payment by user id', function (t) {
  var tqb = new _TabulationQueryBuilder2.default();

  tqb.setTable('payment_logs');

  tqb.setMatching({
    field: 'timestamp',
    range: ['2017-01-01 00:00:00', '2017-01-07 23:59:59']
  });

  tqb.setAggregating({
    field: 'price',
    method: 'sum'
  });

  tqb.setIndexing({
    field: 'user_id',
    interval: 1
  });

  t.is(tqb.build(), 'SELECT SUM(`price`) AS "value", FLOOR(`indexed_value_0` / 1) AS "category" FROM (SELECT user_id AS "indexed_value_0", price FROM payment_logs WHERE ("2017-01-01 00:00:00" <= timestamp AND timestamp <= "2017-01-07 23:59:59")) `indexing_table` GROUP BY FLOOR(`indexed_value_0` / 1)');
});

(0, _ava2.default)('Total payment by user id without interval', function (t) {
  var tqb = new _TabulationQueryBuilder2.default();

  tqb.setTable('payment_logs');

  tqb.setMatching({
    field: 'timestamp',
    range: ['2017-01-01 00:00:00', '2017-01-07 23:59:59']
  });

  tqb.setAggregating({
    field: 'price',
    method: 'sum'
  });

  tqb.setIndexing({
    field: 'user_id'
  });

  t.is(tqb.build(), 'SELECT SUM(`price`) AS "value", `indexed_value_0` AS "category" FROM (SELECT user_id AS "indexed_value_0", price FROM payment_logs WHERE ("2017-01-01 00:00:00" <= timestamp AND timestamp <= "2017-01-07 23:59:59")) `indexing_table` GROUP BY `indexed_value_0`');
});

(0, _ava2.default)('Total user count by item name', function (t) {
  var tqb = new _TabulationQueryBuilder2.default();

  tqb.setTable('payment_logs');

  tqb.setMatching({
    field: 'timestamp',
    range: ['2017-01-01 00:00:00', '2017-01-07 23:59:59']
  });

  tqb.setAggregating({
    field: 'user_id',
    method: 'count'
  });

  tqb.setIndexing({
    field: 'item'
  });

  t.is(tqb.build(), 'SELECT COUNT(`user_id`) AS "value", `indexed_value_0` AS "category" FROM (SELECT `indexed_value_0`, `user_id` FROM (SELECT item AS "indexed_value_0", user_id FROM payment_logs WHERE ("2017-01-01 00:00:00" <= timestamp AND timestamp <= "2017-01-07 23:59:59")) `matching_table` GROUP BY `indexed_value_0`, `user_id`) `indexing_table` GROUP BY `indexed_value_0`');
});

(0, _ava2.default)('Total price by item name', function (t) {
  var tqb = new _TabulationQueryBuilder2.default();

  tqb.setTable('payment_logs');

  tqb.setMatching({
    field: 'timestamp',
    range: ['2017-01-01 00:00:00', '2017-01-07 23:59:59']
  });

  tqb.setAggregating({
    field: 'price',
    method: 'sum'
  });

  tqb.setIndexing({
    field: 'item'
  });

  t.is(tqb.build(), 'SELECT SUM(`price`) AS "value", `indexed_value_0` AS "category" FROM (SELECT item AS "indexed_value_0", price FROM payment_logs WHERE ("2017-01-01 00:00:00" <= timestamp AND timestamp <= "2017-01-07 23:59:59")) `indexing_table` GROUP BY `indexed_value_0`');
});

(0, _ava2.default)('Total user count for each day', function (t) {
  var tqb = new _TabulationQueryBuilder2.default();

  tqb.setTable('payment_logs');

  tqb.setMatching({
    field: 'timestamp',
    range: ['2017-01-01 00:00:00', '2017-01-07 23:59:59']
  });

  tqb.setAggregating({
    field: 'user_id',
    method: 'count'
  });

  tqb.setIndexing({
    field: 'timestamp',
    method: 'eachDays'
  });

  t.is(tqb.build(), 'SELECT COUNT(`user_id`) AS "value", `indexed_value_0` AS "category" FROM (SELECT `indexed_value_0`, `user_id` FROM (SELECT DATE_FORMAT(`timestamp`, "%Y-%m-%d") AS "indexed_value_0", user_id FROM payment_logs WHERE ("2017-01-01 00:00:00" <= timestamp AND timestamp <= "2017-01-07 23:59:59")) `matching_table` GROUP BY `indexed_value_0`, `user_id`) `indexing_table` GROUP BY `indexed_value_0`');
});

(0, _ava2.default)('Average price for each day', function (t) {
  var tqb = new _TabulationQueryBuilder2.default();

  tqb.setTable('payment_logs');

  tqb.setMatching({
    field: 'timestamp',
    range: ['2017-01-01 00:00:00', '2017-01-07 23:59:59']
  });

  tqb.setAggregating({
    field: 'price',
    method: 'avg'
  });

  tqb.setIndexing({
    field: 'timestamp',
    method: 'eachWeeks'
  });

  t.is(tqb.build(), 'SELECT AVG(`price`) AS "value", YEARWEEK(`indexed_value_0`,3) AS "category" FROM (SELECT timestamp AS "indexed_value_0", price FROM payment_logs WHERE ("2017-01-01 00:00:00" <= timestamp AND timestamp <= "2017-01-07 23:59:59")) `indexing_table` GROUP BY YEARWEEK(`indexed_value_0`,3)');
});

(0, _ava2.default)('Average price for each day with sqlite', function (t) {
  var tqb = new _TabulationQueryBuilder2.default({ db: 'sqlite' });

  tqb.setTable('payment_logs');

  tqb.setMatching({
    field: 'timestamp',
    range: ['2017-01-01 00:00:00', '2017-01-07 23:59:59']
  });

  tqb.setAggregating({
    field: 'price',
    method: 'avg'
  });

  tqb.setIndexing({
    field: 'timestamp',
    method: 'eachDays'
  });

  t.is(tqb.build(), 'SELECT AVG(`price`) AS "value", strftime("%Y-%m-%d", `indexed_value_0`) AS "category" FROM (SELECT timestamp AS "indexed_value_0", price FROM payment_logs WHERE ("2017-01-01 00:00:00" <= timestamp AND timestamp <= "2017-01-07 23:59:59")) `indexing_table` GROUP BY strftime("%Y-%m-%d", `indexed_value_0`)');
});

(0, _ava2.default)('Aggregating field is segment field and Counting user by days', function (t) {
  var tqb = new _TabulationQueryBuilder2.default({ segment: 'user_id' });

  tqb.setTable('payment_logs');

  tqb.setMatching({
    field: 'timestamp',
    range: ['2017-01-01 00:00:00', '2017-01-07 23:59:59']
  });

  tqb.setAggregating({
    field: 'user_id',
    method: 'count'
  });

  tqb.setIndexing({
    field: 'timestamp',
    method: 'eachDays'
  });

  t.is(tqb.build(), 'SELECT COUNT(`user_id`) AS "value", `indexed_value_0` AS "category", GROUP_CONCAT(DISTINCT `segment_ids`) AS "segment_ids" FROM (SELECT `indexed_value_0`, `user_id` AS "segment_ids", `user_id` FROM (SELECT DATE_FORMAT(`timestamp`, "%Y-%m-%d") AS "indexed_value_0", user_id FROM payment_logs WHERE ("2017-01-01 00:00:00" <= timestamp AND timestamp <= "2017-01-07 23:59:59")) `matching_table` GROUP BY `indexed_value_0`, `user_id`) `indexing_table` GROUP BY `indexed_value_0`');
});

(0, _ava2.default)('Indexing field is segment field and Total price by user_id', function (t) {
  var tqb = new _TabulationQueryBuilder2.default({ segment: 'user_id' });

  tqb.setTable('payment_logs');

  tqb.setMatching({
    field: 'timestamp',
    range: ['2017-01-01 00:00:00', '2017-01-07 23:59:59']
  });

  tqb.setAggregating({
    field: 'price',
    method: 'sum'
  });

  tqb.setIndexing({
    field: 'user_id'
  });

  t.is(tqb.build(), 'SELECT SUM(`price`) AS "value", `indexed_value_0` AS "category", GROUP_CONCAT(DISTINCT `user_id`) AS "segment_ids" FROM (SELECT user_id AS "indexed_value_0", price, user_id FROM payment_logs WHERE ("2017-01-01 00:00:00" <= timestamp AND timestamp <= "2017-01-07 23:59:59")) `indexing_table` GROUP BY `indexed_value_0`');
});

(0, _ava2.default)('Segment field is not aggregating and indexing fields, and Counting item by days', function (t) {
  var tqb = new _TabulationQueryBuilder2.default({ segment: 'user_id' });

  tqb.setTable('payment_logs');

  tqb.setMatching({
    field: 'timestamp',
    range: ['2017-01-01 00:00:00', '2017-01-07 23:59:59']
  });

  tqb.setAggregating({
    field: 'item',
    method: 'count'
  });

  tqb.setIndexing({
    field: 'timestamp',
    method: 'eachDays'
  });

  t.is(tqb.build(), 'SELECT COUNT(`item`) AS "value", `indexed_value_0` AS "category", GROUP_CONCAT(DISTINCT `segment_ids`) AS "segment_ids" FROM (SELECT `indexed_value_0`, GROUP_CONCAT(`user_id`) AS "segment_ids", `item` FROM (SELECT DATE_FORMAT(`timestamp`, "%Y-%m-%d") AS "indexed_value_0", item, user_id FROM payment_logs WHERE ("2017-01-01 00:00:00" <= timestamp AND timestamp <= "2017-01-07 23:59:59")) `matching_table` GROUP BY `indexed_value_0`, `item`) `indexing_table` GROUP BY `indexed_value_0`');
});

(0, _ava2.default)('Segment field is not aggregating and indexing fields, and Counting item by time of hour', function (t) {
  var tqb = new _TabulationQueryBuilder2.default({ segment: 'user_id' });

  tqb.setTable('payment_logs');

  tqb.setMatching({
    field: 'timestamp',
    range: ['2017-01-01 00:00:00', '2017-01-07 23:59:59']
  });

  tqb.setAggregating({
    field: 'price',
    method: 'avg'
  });

  tqb.setIndexing({
    field: 'timestamp',
    method: 'eachTimeOfHour'
  });

  t.is(tqb.build(), 'SELECT AVG(`price`) AS "value", DATE_FORMAT(`indexed_value_0`, "%H") AS "category", GROUP_CONCAT(DISTINCT `user_id`) AS "segment_ids" FROM (SELECT timestamp AS "indexed_value_0", price, user_id FROM payment_logs WHERE ("2017-01-01 00:00:00" <= timestamp AND timestamp <= "2017-01-07 23:59:59")) `indexing_table` GROUP BY DATE_FORMAT(`indexed_value_0`, "%H")');
});

(0, _ava2.default)('Json item grouping', function (t) {
  var tqb = new _TabulationQueryBuilder2.default();

  tqb.setTable('actions');

  tqb.setMatching({
    field: 'timestamp',
    range: ['2017-01-01 00:00:00', '2017-01-07 23:59:59']
  });

  tqb.setAggregating({
    field: 'user_id',
    method: 'count'
  });

  tqb.setIndexing({
    field: 'detail->"$.view"'
  });

  t.is(tqb.build(), 'SELECT COUNT(`user_id`) AS "value", `indexed_value_0` AS "category" FROM (SELECT `indexed_value_0`, `user_id` FROM (SELECT detail->"$.view" AS "indexed_value_0", user_id FROM actions WHERE ("2017-01-01 00:00:00" <= timestamp AND timestamp <= "2017-01-07 23:59:59")) `matching_table` GROUP BY `indexed_value_0`, `user_id`) `indexing_table` GROUP BY `indexed_value_0`');
});

(0, _ava2.default)('Unique User by day and hours', function (t) {
  var tqb = new _TabulationQueryBuilder2.default();

  tqb.setTable('actions');

  tqb.setMatching({
    field: 'timestamp',
    range: ['2017-01-01 00:00:00', '2017-01-07 23:59:59']
  });

  tqb.setAggregating({
    field: 'user_id',
    method: 'count'
  });

  tqb.setIndexings([{
    field: 'timestamp',
    method: 'eachTimeOfHour'
  }, {
    field: 'timestamp',
    method: 'eachDays'
  }]);

  t.is(tqb.build(), 'SELECT COUNT(`user_id`) AS "value", `indexed_value_0` AS "category_0", `indexed_value_1` AS "category_1" FROM (SELECT `indexed_value_0`, `indexed_value_1`, `user_id` FROM (SELECT DATE_FORMAT(`timestamp`, "%H") AS "indexed_value_0", DATE_FORMAT(`timestamp`, "%Y-%m-%d") AS "indexed_value_1", user_id FROM actions WHERE ("2017-01-01 00:00:00" <= timestamp AND timestamp <= "2017-01-07 23:59:59")) `matching_table` GROUP BY `indexed_value_0`, `indexed_value_1`, `user_id`) `indexing_table` GROUP BY `indexed_value_0`, `indexed_value_1`');
});

(0, _ava2.default)('Top sales by item and time of hours', function (t) {
  var tqb = new _TabulationQueryBuilder2.default();

  tqb.setTable('payment_logs');

  tqb.setMatching({
    field: 'timestamp',
    range: ['2017-01-01 00:00:00', '2017-01-07 23:59:59']
  });

  tqb.setAggregating({
    field: 'price',
    method: 'sum'
  });

  tqb.setIndexings([{
    field: 'timestamp',
    method: 'eachTimeOfHour'
  }, {
    field: 'item'
  }]);

  t.is(tqb.build(), 'SELECT SUM(`price`) AS "value", DATE_FORMAT(`indexed_value_0`, "%H") AS "category_0", `indexed_value_1` AS "category_1" FROM (SELECT timestamp AS "indexed_value_0", item AS "indexed_value_1", price FROM payment_logs WHERE ("2017-01-01 00:00:00" <= timestamp AND timestamp <= "2017-01-07 23:59:59")) `indexing_table` GROUP BY DATE_FORMAT(`indexed_value_0`, "%H"), `indexed_value_1`');
});

(0, _ava2.default)('Timezone convert option with matching and indexing', function (t) {
  var tqb = new _TabulationQueryBuilder2.default({
    timezone: {
      from: 'UTC',
      to: 'Asia/Tokyo'
    }
  });

  tqb.setTable('payment_logs');

  tqb.setMatching({
    field: 'timestamp',
    range: ['2017-01-01 00:00:00', '2017-01-07 23:59:59'],
    type: 'datetime'
  });

  tqb.setAggregating({
    field: 'user_id',
    method: 'count'
  });

  tqb.setIndexing({
    field: 'timestamp',
    method: 'eachDays'
  });

  t.is(tqb.build(), 'SELECT COUNT(`user_id`) AS "value", `indexed_value_0` AS "category" FROM (SELECT `indexed_value_0`, `user_id` FROM (SELECT DATE_FORMAT(CONVERT_TZ(`timestamp`, "UTC", "Asia/Tokyo"), "%Y-%m-%d") AS "indexed_value_0", user_id FROM payment_logs WHERE ("2017-01-01 00:00:00" <= CONVERT_TZ(timestamp, "UTC", "Asia/Tokyo") AND CONVERT_TZ(timestamp, "UTC", "Asia/Tokyo") <= "2017-01-07 23:59:59")) `matching_table` GROUP BY `indexed_value_0`, `user_id`) `indexing_table` GROUP BY `indexed_value_0`');
});

(0, _ava2.default)('Unixtime and Timezone convert option with matching and indexing', function (t) {
  var tqb = new _TabulationQueryBuilder2.default({
    timezone: {
      from: 'UTC',
      to: 'Asia/Tokyo'
    }
  });

  tqb.setTable('payment_logs');

  tqb.setMatching({
    field: 'timestamp',
    range: ['2017-01-01 00:00:00', '2017-01-07 23:59:59'],
    type: 'unixtime'
  });

  tqb.setAggregating({
    field: 'user_id',
    method: 'count'
  });

  tqb.setIndexing({
    field: 'timestamp',
    method: 'eachDays',
    type: 'unixtime'
  });

  t.is(tqb.build(), 'SELECT COUNT(`user_id`) AS "value", `indexed_value_0` AS "category" FROM (SELECT `indexed_value_0`, `user_id` FROM (SELECT DATE_FORMAT(CONVERT_TZ(FROM_UNIXTIME(`timestamp`), "UTC", "Asia/Tokyo"), "%Y-%m-%d") AS "indexed_value_0", user_id FROM payment_logs WHERE ("2017-01-01 00:00:00" <= CONVERT_TZ(FROM_UNIXTIME(timestamp), "UTC", "Asia/Tokyo") AND CONVERT_TZ(FROM_UNIXTIME(timestamp), "UTC", "Asia/Tokyo") <= "2017-01-07 23:59:59")) `matching_table` GROUP BY `indexed_value_0`, `user_id`) `indexing_table` GROUP BY `indexed_value_0`');
});