'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _squel = require('squel');

var _squel2 = _interopRequireDefault(_squel);

var _StringDecorator = require('../utils/StringDecorator');

var _TimeUtils = require('../utils/TimeUtils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var AggregatingConfig = function () {
  function AggregatingConfig(config) {
    (0, _classCallCheck3.default)(this, AggregatingConfig);

    this.config = config;
  }

  AggregatingConfig.prototype.getCategoryField = function getCategoryField(length, index) {
    if (length > 1) {
      return 'category_' + index;
    }

    return 'category';
  };

  AggregatingConfig.prototype.addSegmentField = function addSegmentField(query) {
    if (!this.config.segment) {
      return query;
    }

    if (this.config.method === 'count') {
      query.field('GROUP_CONCAT(DISTINCT ' + (0, _StringDecorator.addBacktick)('segment_ids') + ')', 'segment_ids');
    } else {
      query.field('GROUP_CONCAT(DISTINCT ' + (0, _StringDecorator.addBacktick)(this.config.segment) + ')', 'segment_ids');
    }

    return query;
  };

  AggregatingConfig.prototype.build = function build(table, indexings) {
    var _this = this;

    var method = this.config.method.toUpperCase();

    var query = _squel2.default.select().field(method + '(' + (0, _StringDecorator.addBacktick)(this.field) + ')', 'value');

    indexings.forEach(function (indexing, index) {
      if (indexing.method && indexing.method.match(/each/)) {
        return _this.parseWithEachDate(query, table, method, indexing, index, indexings.length);
      }

      if (indexing.interval) {
        return _this.parseWithInterval(query, table, method, indexing, index, indexings.length);
      }

      if (indexing.categoryRange && indexing.categoryRange.length < 2) {
        throw new Error('Invalid categoryRange. categoryRange must be more than 2');
      }

      if (indexing.categoryRange) {
        return _this.parseWithCategoryRange(query, table, method, indexing, index, indexings.length);
      }

      _this.parse(query, table, method, indexing, index, indexings.length);
    });

    this.addSegmentField(query);

    query.from(table, 'indexing_table');

    return query;
  };

  AggregatingConfig.prototype.parse = function parse(query, table, method, indexingConfig, index, indexingsLength) {
    var indexedValue = (0, _StringDecorator.addBacktick)((0, _StringDecorator.getIndexedValue)(index));

    query.field(indexedValue, this.getCategoryField(indexingsLength, index));

    query.group('' + indexedValue);

    return query;
  };

  AggregatingConfig.prototype.parseWithEachDate = function parseWithEachDate(query, table, method, indexingConfig, index, indexingsLength) {
    var indexedValue = (0, _StringDecorator.addBacktick)((0, _StringDecorator.getIndexedValue)(index));

    var term = (0, _TimeUtils.getDateFormatQuery)(this.config.db, indexedValue, indexingConfig.method, this.config.timezone, indexingConfig.type);

    query.field(term, this.getCategoryField(indexingsLength, index));

    query.group(term);

    return query;
  };

  AggregatingConfig.prototype.parseWithInterval = function parseWithInterval(query, table, method, indexingConfig, index, indexingsLength) {
    var indexedValue = (0, _StringDecorator.addBacktick)((0, _StringDecorator.getIndexedValue)(index));

    var intervalString = 'FLOOR(' + indexedValue + ' / ' + indexingConfig.interval + ')';

    query.field(intervalString, this.getCategoryField(indexingsLength, index));

    query.group(intervalString);

    return query;
  };

  AggregatingConfig.prototype.parseWithCategoryRange = function parseWithCategoryRange(query, table, method, indexingConfig, index, indexingsLength) {
    var indexedValue = (0, _StringDecorator.addBacktick)((0, _StringDecorator.getIndexedValue)(index));

    var categoryRange = indexingConfig.categoryRange;

    var categoryRangeQuery = _squel2.default.case();

    categoryRange.forEach(function (category, i) {
      if (i === categoryRange.length - 1) {
        return;
      }

      categoryRangeQuery.when(category + ' <= ' + indexedValue + ' AND ' + indexedValue + ' < ' + categoryRange[i + 1]).then(category);
    });

    query.field(categoryRangeQuery, this.getCategoryField(indexingsLength, index));

    query.group(categoryRangeQuery);

    return query;
  };

  (0, _createClass3.default)(AggregatingConfig, [{
    key: 'field',
    get: function get() {
      return this.config.field;
    }
  }, {
    key: 'method',
    get: function get() {
      return this.config.method;
    }
  }]);
  return AggregatingConfig;
}();

exports.default = AggregatingConfig;