'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _squel = require('squel');

var _squel2 = _interopRequireDefault(_squel);

var _StringDecorator = require('../utils/StringDecorator');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var IndexingConfig = function () {
  function IndexingConfig(config) {
    (0, _classCallCheck3.default)(this, IndexingConfig);

    this.config = config;
  }

  IndexingConfig.build = function build(table, aggregating, indexings, segment) {
    var aggregatingString = (0, _StringDecorator.addBacktick)(aggregating.field);

    var query = _squel2.default.select();

    var isAggregatingGrouping = false;

    indexings.forEach(function (indexing, index) {
      var isNeedGrouping = (!indexing.method || indexing.method && indexing.method.match(/each/)) && aggregating.method === 'count';

      var indexedField = (0, _StringDecorator.addBacktick)((0, _StringDecorator.getIndexedValue)(index));

      if (indexing.method && !indexing.method.match(/each/)) {
        var method = indexing.method.toUpperCase();

        query.field(method + '(' + indexedField + ')', (0, _StringDecorator.getIndexedValue)(index));
      } else {
        query.field(indexedField);
      }

      if (segment && isNeedGrouping && segment !== aggregating.field && !indexings.find(function (i) {
        return i.field === segment;
      })) {
        query.field('GROUP_CONCAT(' + (0, _StringDecorator.addBacktick)(segment) + ')', 'segment_ids');
      } else if (segment) {
        query.field((0, _StringDecorator.addBacktick)(segment), 'segment_ids');
      }

      if (isNeedGrouping) {
        isAggregatingGrouping = true;

        query.group((0, _StringDecorator.addBacktick)((0, _StringDecorator.getIndexedValue)(index)));
      } else if (indexing.method) {
        isAggregatingGrouping = true;
      }
    });

    if (isAggregatingGrouping) {
      query.group(aggregatingString);
    }

    query.field(aggregatingString);

    query.from(table, 'matching_table');

    return query;
  };

  (0, _createClass3.default)(IndexingConfig, [{
    key: 'field',
    get: function get() {
      return this.config.field;
    }
  }, {
    key: 'method',
    get: function get() {
      return this.config.method;
    }
  }, {
    key: 'interval',
    get: function get() {
      return this.config.interval;
    }
  }, {
    key: 'type',
    get: function get() {
      return this.config.type;
    }
  }, {
    key: 'categoryRange',
    get: function get() {
      return this.config.categoryRange;
    }
  }]);
  return IndexingConfig;
}();

exports.default = IndexingConfig;