'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _squel = require('squel');

var _squel2 = _interopRequireDefault(_squel);

var _TimeUtils = require('../utils/TimeUtils');

var _StringDecorator = require('../utils/StringDecorator');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var MatchingConfig = function () {
  function MatchingConfig(config) {
    (0, _classCallCheck3.default)(this, MatchingConfig);

    this.config = config;
  }

  MatchingConfig.prototype.build = function build(table, indexings, aggregating) {
    var _this = this;

    var query = _squel2.default.select();

    indexings.forEach(function (indexing, index) {
      if (indexing.method && indexing.method.match(/each/) && aggregating.method === 'count') {
        var dateFormatField = (0, _TimeUtils.getDateFormatQuery)(_this.config.db, (0, _StringDecorator.addBacktick)(indexing.field), indexing.method, _this.config.timezone, indexing.type);

        query.field(dateFormatField, (0, _StringDecorator.getIndexedValue)(index));
      } else {
        query.field(indexing.field, (0, _StringDecorator.getIndexedValue)(index));
      }
    });

    query.field(aggregating.field);

    if (this.config.segment && this.config.segment !== aggregating.field) {
      query.field(this.config.segment);
    }

    query.from(table).where(this.parse(_squel2.default, this.config));

    return query;
  };

  MatchingConfig.prototype.parse = function parse(builder, conf) {
    if (conf.field === undefined) {
      return this.parseTerm(builder, conf);
    }

    if (conf.range) {
      return this.parseRange(conf.field, conf.range, conf.type);
    }

    if (conf.in) {
      return this.parseIn(conf.field, conf.in);
    }

    if (conf.operator) {
      return this.parseOperator(conf.field, conf.operator, conf.value, conf.type);
    }
  };

  MatchingConfig.prototype.parseIn = function parseIn(field, inStatement) {
    return field + ' IN (' + inStatement.map(function (statement) {
      return '"' + statement + '"';
    }).join(',') + ')';
  };

  MatchingConfig.prototype.parseRange = function parseRange(field, range, type) {
    var rangeField = field;

    if (type === 'datetime' || type === 'unixtime') {
      rangeField = (0, _TimeUtils.getTimezoneConvertedDatetime)(field, this.config.timezone, type);
    }

    return '"' + range[0] + '" <= ' + rangeField + ' AND ' + rangeField + ' <= "' + range[1] + '"';
  };

  MatchingConfig.prototype.parseOperator = function parseOperator(field, operator, value, type) {
    var rangeField = field;

    if (type === 'datetime' || type === 'unixtime') {
      rangeField = (0, _TimeUtils.getTimezoneConvertedDatetime)(field, this.config.timezone, type);
    }

    return rangeField + ' ' + operator + ' "' + value + '"';
  };

  MatchingConfig.prototype.parseTerm = function parseTerm(builder, conf) {
    var operator = conf.and ? 'and' : 'or';

    if (!conf[operator]) {
      throw new Error('Invalid matching config. No field \'and\', \'or\'. ' + (0, _stringify2.default)(conf));
    }

    return builder.expr()[operator](this.parse(builder, conf[operator][0]))[operator](this.parse(builder, conf[operator][1]));
  };

  return MatchingConfig;
}();

exports.default = MatchingConfig;