'use strict';

var _ava = require('ava');

var _ava2 = _interopRequireDefault(_ava);

var _MatchingConfig = require('./MatchingConfig');

var _MatchingConfig2 = _interopRequireDefault(_MatchingConfig);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

(0, _ava2.default)('Date range expression', function (t) {
  var config = new _MatchingConfig2.default({
    field: 'timestamp',
    range: ['2017-01-01 00:00:00', '2017-01-03 23:59:59']
  });

  t.is(config.build('payment_logs', [{ field: 'user_id' }], { field: 'price' }).toString(), 'SELECT user_id AS "indexed_value_0", price FROM payment_logs WHERE ("2017-01-01 00:00:00" <= timestamp AND timestamp <= "2017-01-03 23:59:59")');
});

(0, _ava2.default)('Date range or expression', function (t) {
  var config = new _MatchingConfig2.default({
    or: [{
      field: 'timestamp',
      range: ['2017-01-01 00:00:00', '2017-01-01 23:59:59']
    }, {
      field: 'timestamp',
      range: ['2017-01-03 00:00:00', '2017-01-03 23:59:59']
    }]
  });

  t.is(config.build('payment_logs', [{ field: 'user_id' }], { field: 'price' }).toString(), 'SELECT user_id AS "indexed_value_0", price FROM payment_logs WHERE ("2017-01-01 00:00:00" <= timestamp AND timestamp <= "2017-01-01 23:59:59" OR "2017-01-03 00:00:00" <= timestamp AND timestamp <= "2017-01-03 23:59:59")');
});

(0, _ava2.default)('String in expression', function (t) {
  var config = new _MatchingConfig2.default({
    and: [{
      field: 'user_id',
      in: [1, 2, 3]
    }, {
      or: [{
        field: 'timestamp',
        range: ['2017-01-01 00:00:00', '2017-01-01 23:59:59']
      }, {
        field: 'timestamp',
        range: ['2017-01-03 00:00:00', '2017-01-03 23:59:59']
      }]
    }]
  });

  t.is(config.build('payment_logs', [{ field: 'user_id' }], { field: 'price' }).toString(), 'SELECT user_id AS "indexed_value_0", price FROM payment_logs WHERE (user_id IN ("1","2","3") AND ("2017-01-01 00:00:00" <= timestamp AND timestamp <= "2017-01-01 23:59:59" OR "2017-01-03 00:00:00" <= timestamp AND timestamp <= "2017-01-03 23:59:59"))');
});

(0, _ava2.default)('operator and value in expression', function (t) {
  var config = new _MatchingConfig2.default({
    and: [{
      field: 'user_id',
      in: [1, 2, 3]
    }, {
      and: [{
        field: 'timestamp',
        operator: '>=',
        value: '2017-01-01 00:00:00'
      }, {
        field: 'timestamp',
        operator: '<',
        value: '2017-01-03 23:59:59'
      }]
    }]
  });

  t.is(config.build('payment_logs', [{ field: 'user_id' }], { field: 'price' }).toString(), 'SELECT user_id AS "indexed_value_0", price FROM payment_logs WHERE (user_id IN ("1","2","3") AND (timestamp >= "2017-01-01 00:00:00" AND timestamp < "2017-01-03 23:59:59"))');
});