'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getDateFormatQuery = getDateFormatQuery;
exports.getTimezoneConvertedDatetime = getTimezoneConvertedDatetime;
function getDateFormatQuery(db, field, method, timezone, type) {
  var isSqlite = db === 'sqlite';

  var timeField = getTimezoneConvertedDatetime(field, timezone, type);

  switch (method) {
    case 'eachDays':
      return isSqlite ? 'strftime("%Y-%m-%d", ' + timeField + ')' : 'DATE_FORMAT(' + timeField + ', "%Y-%m-%d")';
    case 'eachWeeks':
      return isSqlite ? 'strftime("%Y%W", ' + timeField + ')' : 'YEARWEEK(' + timeField + ',3)';
    case 'eachMonths':
      return isSqlite ? 'strftime("%Y-%m", ' + timeField + ')' : 'DATE_FORMAT(' + timeField + ', "%Y-%m")';
    case 'eachYears':
      return isSqlite ? 'strftime("%Y", ' + timeField + ')' : 'DATE_FORMAT(' + timeField + ', "%Y")';
    case 'eachDayOfWeek':
      return isSqlite ? 'strftime("%w", ' + timeField + ')' : 'DATE_FORMAT(' + timeField + ', "%w")';
    case 'eachTimeOfHour':
      return isSqlite ? 'strftime("%H", ' + timeField + ')' : 'DATE_FORMAT(' + timeField + ', "%H")';
  }
}

function getTimezoneConvertedDatetime(field, timezone, type) {
  var typedField = field;

  if (type === 'unixtime') {
    typedField = 'FROM_UNIXTIME(' + typedField + ')';
  }

  if (timezone) {
    return 'CONVERT_TZ(' + typedField + ', "' + timezone.from + '", "' + timezone.to + '")';
  }

  return typedField;
}